/**
 * Copyright 2007-2013 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.maven.jbuilding;

import java.io.File;
import java.util.Collections;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.DefaultArtifact;
import org.apache.maven.artifact.handler.DefaultArtifactHandler;
import org.apache.maven.artifact.resolver.ArtifactResolutionRequest;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.ResolutionNode;
import org.apache.maven.plugin.testing.AbstractMojoTestCase;
import org.apache.maven.repository.RepositorySystem;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.ow2.util.file.FileUtils;
import org.ow2.util.maven.jbuilding.stubs.TestableMavenProject;
import org.ow2.util.plan.bindings.deploymentplan.Deployment;
import org.ow2.util.plan.bindings.deploymentplan.maven2.Maven2Deployment;
import org.ow2.util.plan.reader.plan.IPlanReader;
import org.ow2.util.plan.reader.plan.PlanReaderImpl;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

/**
 * A {@code ConditionalDPGenerationTestCase} tests generation of conditional deployment plans
 *
 * @author Loic Albertin
 */
public class ConditionalDPGenerationTestCase extends AbstractMojoTestCase {

    @Mock
    private RepositorySystem repositorySystem;

    private Artifact dummyArtifact =
            new DefaultArtifact("org.ow2.util.plan", "plan-reader", "2.0", null, "jar", null, new DefaultArtifactHandler("jar"));

    @Mock
    private ArtifactResolutionResult artifactResolutionResult;

    private String outputDirectory;

    @Override
    protected void setUp() throws Exception {
        // required for mojo lookups to work
        super.setUp();
        MockitoAnnotations.initMocks(this);
        outputDirectory = getBasedir() + File.separator + "target" + File.separator + "configuration-resources";
    }

    /**
     * Tests generation of a deployment plan with a condition on the whole deployment plan
     * @throws Exception if anything goes wrong
     */
    public void testConditionGenerationOnDeploymentPlanItem() throws Exception {

        String filename =
                outputDirectory + File.separator + "repositories" +
                        File.separator + "url-internal" + File.separator + "test-condition-on-dp.xml";
        FileUtils.delete(filename);


        GenerateMaven2DeploymentPlansMojo plansMojo = getMojo("condition-deployment-plan-generation.pom");

        plansMojo.setOutput(new File(outputDirectory));

        plansMojo.execute();
        File generatedDP = new File(filename);
        assertTrue("Generated DeploymentPlan doesn't exist", generatedDP.exists());

        IPlanReader planReader = new PlanReaderImpl();
        org.ow2.util.plan.bindings.deploymentplan.DeploymentPlan deploymentPlan = planReader.readPlan(generatedDP);

        assertNotNull("Condition on deployment plan doesn't exist", deploymentPlan.getCondition());
        assertEquals("!(systemProperty('disableDP').set && systemProperty('disableDP').value=='true')", deploymentPlan.getCondition());


        for (Deployment deployment : deploymentPlan.getDeployments()) {
            assertNull("Only the Deployment Plan should have a condition not inner deployments. ", deployment.getCondition());
        }
    }

    /**
     * Tests generation of a deployment plan with a condition on an artifact item
     * @throws Exception if anything goes wrong
     */
    public void testConditionGenerationOnArtifactItem() throws Exception {

        String filename =
                outputDirectory + File.separator + "repositories" +
                        File.separator + "url-internal" + File.separator + "test-condition-on-artifact.xml";
        FileUtils.delete(filename);


        GenerateMaven2DeploymentPlansMojo plansMojo = getMojo("condition-artifact-generation.pom");

        plansMojo.setOutput(new File(outputDirectory));

        plansMojo.execute();
        File generatedDP = new File(filename);
        assertTrue("Generated DeploymentPlan doesn't exist", generatedDP.exists());

        IPlanReader planReader = new PlanReaderImpl();
        org.ow2.util.plan.bindings.deploymentplan.DeploymentPlan deploymentPlan = planReader.readPlan(generatedDP);

        assertNull("Condition on deployment should be null for this test.", deploymentPlan.getCondition());

        String condition = null;
        for (Deployment deployment : deploymentPlan.getDeployments()) {
            if (deployment instanceof Maven2Deployment) {
                condition = deployment.getCondition();
            }
        }
        assertNotNull("Condition not found", condition);
        assertEquals("!(systemProperty('disableDP').set && systemProperty('disableDP').value=='true')", condition);

    }


    /**
     * Retrieve a {@link GenerateMaven2DeploymentPlansMojo} for a given pom
     * @param pomName The pom name
     * @return a {@link GenerateMaven2DeploymentPlansMojo}
     * @throws Exception if retrieval fails
     */
    private GenerateMaven2DeploymentPlansMojo getMojo(String pomName) throws Exception {
        File pom = new File(getBasedir(), "src/test/resources/org/ow2/util/maven/jbuilding/" + pomName);
        when(repositorySystem.createArtifactWithClassifier(anyString(), anyString(), anyString(), anyString(), anyString()))
                .thenReturn(dummyArtifact);
        when(repositorySystem.resolve(any(ArtifactResolutionRequest.class)))
                .thenReturn(artifactResolutionResult);
        when(artifactResolutionResult.isSuccess())
                .thenReturn(Boolean.TRUE);
        when(artifactResolutionResult.getArtifacts()).thenReturn(Collections.singleton(dummyArtifact));
        when(artifactResolutionResult.getArtifactResolutionNodes())
                .thenReturn(Collections.singleton(new ResolutionNode(dummyArtifact, null)));

        getContainer().addComponent(repositorySystem, RepositorySystem.class, null);

        GenerateMaven2DeploymentPlansMojo mojo = (GenerateMaven2DeploymentPlansMojo) lookupConfiguredMojo(
                new TestableMavenProject(pom),
                "generate-maven2-deployment-plans"
        );

        assertNotNull("Can't retrieve mojo.",mojo);
        return mojo;
    }
}
